#!/usr/bin/env python

paletteName = "Sample"

#redRange = [0.0, 0.5, 0.75, 1.0]
#redRange = [0.0, 0.2, 0.4, 0.6, 0.8, 1.0]
#redRange = [0.0, 0.25, 0.5, 0.75, 0.9, 1.0]
redRange = [0.0, 0.25, 0.5, 0.625, 0.75, 0.875, 1.0]
greenRange = redRange
blueRange = redRange

hueRange = range(0,360,30)
#satRange = [0, 0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9, 1.0]
satRange = [0, 0.25, 0.5, 0.75, 1.0]
#satRange = [0.0, 0.5, 1.0]
#lightRange = [0, 0.1667, 0.3333, 0.5, 0.6667, 0.8333, 1.0]
lightRange = [0, 0.20, 0.35, 0.5, 0.65, 0.80, 1.0]
#lightRange = [0, 0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9, 1.0]
#lightRange = [0, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 1.0]
#lightRange = [0.0, 0.5, 1.0]
#lightRange = [0, 0.2, 0.4, 0.5, 0.6, 0.8, 1.0]
#greyRange = [0, 0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9, 1.0]


import string
import math

numBits = 4

def convertIntRGBtoPalette((r,g,b)):
	"""Converts the RGB colour to one supported by the palette I want"""
	(r,g,b) = (int(r),int(g),int(b))
	# return convertIntRGBtoBit((r,g,b),numBits)
	return (r,g,b)

def convertIntRGBtoBit((r,g,b), nBits):
	k = 256.0 / (pow(2,nBits)-1)
	r = min(255, int(round(r / k) * k))
	g = min(255, int(round(g / k) * k))
	b = min(255, int(round(b / k) * k))
	return (r,g,b)
	
def convertHSLtoRGB((h,s,l)):
	"Converts an HSL colour to RGB coords"""
	if s == 0:
		return (l,l,l)
	if (l < 0.5):
		temp2 = l*(1.0+s)
	else:
		temp2 = l + s - l*s
	
	temp1 = 2.0 * l - temp2
	
	# h should already be in the range 0 to 1
	
	def calcColour(val):
		if val < 0:
			val = val + 1.0
		if val > 1:
			val = val - 1.0
		if 6.0 * val < 1:
			val = temp1 + (temp2 - temp1) * 6.0 * val
		elif 2.0 * val < 1:
			val = temp2
		elif 3.0 * val < 2:
			val = temp1 + (temp2 - temp1) * ((2.0/3.0) - val) * 6.0
		else:
			val = temp1
		return val
	
	r = calcColour(h + 1.0/3.0)
	g = calcColour(h)
	b = calcColour(h - 1.0/3.0)
	
	return (r,g,b)
	
	
	
	
def convertRGBtoHSL((r,g,b)):
	"""Figure out the HSL from provided RGB coords"""
	cMax = max(r,g,b)
	cMin = min(r,g,b)
	# figure out the hue
	hue = 0
	if (cMax == cMin):
		hue = 0
	elif (cMax == r):
		hue = 60 * (g-b)/(cMax-cMin)
		if g < b:
			hue = hue + 360
	elif (cMax == g):
		hue = 60 * (b-r)/(cMax-cMin) + 120
	else:
		hue = 60 * (r-g)/(cMax-cMin) + 240
		
	# figure out the lightness
	light = (cMax + cMin) / 2.0
	
	# figure out the saturation
	sat = 0
	if (cMax != cMin) and light < 1.0:
		if (light < 0.5):
			sat = (cMax - cMin) / (2*light)
		else:
			sat = (cMax - cMin) / (2 - 2*light)
	
	# convert the hue back to the 0 to 1 range
	hue = hue / 360
	
	return (hue, sat, light)

def convertFloatRGBtoIntRGB((r,g,b)):
	r = int(round(r * 255.0))
	g = int(round(g * 255.0))
	b = int(round(b * 255.0))
	r = min(255, r)
	g = min(255, g)
	b = min(255, b)
	r = max(0, r)
	g = max(0, g)
	b = max(0, b)
	return (r,g,b)
		
def colourOldName((r,g,b)):
	"""Return a string that represents this colour (depreciated)"""
	(hue, sat, light) = convertRGBtoHSL((r/255.0,g/255.0,b/255.0))
	hueString = "H:%d" % int(hue*360)
	satString = "S:%d" % int(sat*100)
	lightString = "L:%d" % int(light*100)
	return "%s,%s,%s" % (hueString, satString, lightString) 

def colourName((r,g,b)):
	"""Return a string that represents this colour"""
	(hue, sat, light) = convertRGBtoHSL((r/255.0,g/255.0,b/255.0))
	hueLetters = "ROYLGSCABVMPR"
	hueIndex= int(round(hue / (1.0/12.0)))
	hueLetter = hueLetters[hueIndex]
	
	satValue = int(round((1 - sat) * 20))
	lightValue = int(round(light*20))-10
	if (lightValue <= -10):
		return "K"
	if (lightValue >= 10):
		return "W"
	satLetter = ""
	if (satValue >= 20):		
		hueLetter = 'N'
	elif (satValue > 0):
		satLetters = "abcdefghijklmnopqrs"
		satLetter = satLetters[satValue-1]
	
	if lightValue < 0:
		return str(-lightValue) + hueLetter + satLetter
	elif lightValue > 0:
		return hueLetter + str(lightValue) + satLetter
	else:
		return hueLetter + satLetter
	
	return "%s,%s,%s" % (hueString, satString, lightString) 

def colourSortFunc((r1,g1,b1),(r2,g2,b2)):
	"""Returns how to sort these two elements"""
	#val1 = int(round((r1-r2)*255))
	#val2 = int(round((g1-g2)*255))
	#val3 = int(round((b1-b2)*255))
	(h1,s1,l1) = convertRGBtoHSL((r1/255.0,g1/255.0,b1/255.0))
	(h2,s2,l2) = convertRGBtoHSL((r2/255.0,g2/255.0,b2/255.0))
	if s1 == 0 and s2 != 0:
		return 1
	if s1 != 0 and s2 == 0:
		return -1
	if s1 == 0 and s2 == 0:
		if l1 > l2:
			return 1
		elif l1 < l2:
			return -1
		else:
			return 0
		
	val1 = int(round((h1-h2)*36))
	val2 = -int(round((s1-s2)*10))
	val3 = int(round((l1-l2)*100))
			
	if (val1 != 0):
		return val1
	if (val2 != 0):
		return val2
	if (val3 != 0):
		return val3
	return 0




colourList = []
for h in hueRange:
	for s in satRange:
		for l in lightRange:
			colourList.append((h/360.0,s,l))

colourList = [convertHSLtoRGB(elem) for elem in colourList]
colourList = [convertFloatRGBtoIntRGB(elem) for elem in colourList]

#for g in greyRange:
#	grey = (g, g, g)
#	colourList.append(convertFloatRGBtoIntRGB(grey))

colourList = list(set(colourList))
colourList.sort(cmp=colourSortFunc)

#colourList = [convertIntRGBtoPalette(elem) for elem in colourList]

#colourList = list(set(colourList))
#colourList.sort(cmp=colourSortFunc)

print "GIMP Palette"
print "Name: %s" % paletteName
#if len(greenRange) % 2:
#	print "Columns: %d" % len(greenRange)
#else:
#	print "Columns: %d" % (len(greenRange)*2)
#print "Columns: %d" % int(math.sqrt(len(colourList)))
print "Columns: %d" % ((len(lightRange) - 2) * (len(satRange) - 1))
print "#"
print "# Automatically built using a Python script: David Shaw"
print "#"


for (r, g, b) in colourList:
	name = colourName((r, g, b))
	redStr = string.rjust(str(r),3)
	greenStr = string.rjust(str(g),3)
	blueStr = string.rjust(str(b),3)
	print "%s %s %s\t%s" % (redStr, greenStr, blueStr, name)



